@@ -44,7 +44,7 @@
 #include <string.h>
 
 #include "gstflacdec.h"
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 #include <gst/tag/tag.h>
 
 #include "gstflacelements.h"
