@@ -25,6 +25,9 @@
 #include <jack/jack.h>
 #include <gst/audio/audio.h>
 
+GST_ELEMENT_REGISTER_DECLARE (jackaudiosrc);
+GST_ELEMENT_REGISTER_DECLARE (jackaudiosink);
+
 /**
  * GstJackConnect:
  * @GST_JACK_CONNECT_NONE: Don't automatically connect to physical ports.
@@ -42,7 +45,17 @@
 typedef enum {
   GST_JACK_CONNECT_NONE,
   GST_JACK_CONNECT_AUTO,
-  GST_JACK_CONNECT_AUTO_FORCED
+  GST_JACK_CONNECT_AUTO_FORCED,
+
+  /**
+   * GstJackConnect::explicit
+   *
+   * In this mode, the element will try to connect to explicitly requested
+   * port specified by "port-names".
+   *
+   * Since: 1.20
+   */
+  GST_JACK_CONNECT_EXPLICIT,
 } GstJackConnect;
 
 /**
