@@ -60,7 +60,7 @@
 
 #define H264_CAPS \
   "video/x-h264, " \
-  "stream-format = (string) avc, " \
+  "stream-format = (string) { avc, avc3 }, " \
   "alignment = (string) au, " \
   COMMON_VIDEO_CAPS
 
@@ -93,6 +93,12 @@
   "svqversion = (int) 3, " \
   COMMON_VIDEO_CAPS
 
+#define VP9_CAPS \
+  "video/x-vp9, " \
+  "profile = (string) { 0, 1, 2, 3 }, " \
+  "chroma-format = (string) { 4:2:0, 4:2:2, 4:4:4 }, " \
+  COMMON_VIDEO_CAPS
+
 #define COMMON_AUDIO_CAPS(c, r) \
   "channels = (int) [ 1, " G_STRINGIFY (c) " ], " \
   "rate = (int) [ 1, " G_STRINGIFY (r) " ]"
@@ -166,6 +172,11 @@
   "text/x-raw, " \
   "format=(string)utf8"
 
+#define CEA608_CAPS \
+  "closedcaption/x-cea-608, format=(string)s334-1a"
+#define CEA708_CAPS \
+  "closedcaption/x-cea-708, format=(string)cdp"
+
 /* FIXME 0.11 - take a look at bugs #580005 and #340375 */
 GstQTMuxFormatProp gst_qt_mux_format_list[] = {
   /* original QuickTime format; see Apple site (e.g. qtff.pdf) */
@@ -196,20 +207,18 @@ GstQTMuxFormatProp gst_qt_mux_format_list[] = {
             COMMON_VIDEO_CAPS_NO_FRAMERATE "; "
             "video/x-vp8, "
             COMMON_VIDEO_CAPS "; "
-            "video/x-vp9, "
-            COMMON_VIDEO_CAPS "; "
+            VP9_CAPS "; "
             "video/x-dirac, "
-            COMMON_VIDEO_CAPS "; " "video/x-qt-part, " COMMON_VIDEO_CAPS),
-        GST_STATIC_CAPS (PCM_CAPS_FULL "; "
-            PCM_CAPS_UNPOSITIONED " ; "
-            MP123_CAPS " ; "
-            AAC_CAPS " ; "
-            AC3_CAPS " ; "
-            ADPCM_CAPS " ; "
-            "audio/x-alaw, " COMMON_AUDIO_CAPS (2, MAX) "; "
-            "audio/x-mulaw, " COMMON_AUDIO_CAPS (2, MAX) "; "
-            AMR_CAPS " ; " ALAC_CAPS " ; " OPUS_CAPS),
-      GST_STATIC_CAPS (TEXT_UTF8)}
+            COMMON_VIDEO_CAPS "; " "video/x-qt-part, " COMMON_VIDEO_CAPS "; "
+            "video/x-av1, " "stream-format = (string) \"obu-stream\", "
+            "alignment = (string) \"tu\", " COMMON_VIDEO_CAPS ";"),
+        GST_STATIC_CAPS (PCM_CAPS_FULL "; " PCM_CAPS_UNPOSITIONED " ; "
+            MP123_CAPS " ; " AAC_CAPS " ; " AC3_CAPS " ; " ADPCM_CAPS " ; "
+            "audio/x-alaw, " COMMON_AUDIO_CAPS (2,
+                MAX) "; " "audio/x-mulaw, " COMMON_AUDIO_CAPS (2,
+                MAX) "; " AMR_CAPS " ; " ALAC_CAPS " ; " OPUS_CAPS),
+        GST_STATIC_CAPS (TEXT_UTF8),
+      GST_STATIC_CAPS (CEA608_CAPS "; " CEA708_CAPS)}
   ,
   /* ISO 14496-14: mp42 as ISO base media extension
    * (supersedes original ISO 144996-1 mp41) */
@@ -221,10 +230,14 @@ GstQTMuxFormatProp gst_qt_mux_format_list[] = {
         "GstMP4Mux",
         GST_STATIC_CAPS ("video/quicktime, variant = (string) iso"),
         GST_STATIC_CAPS (MPEG4V_CAPS "; " H264_CAPS ";" H265_CAPS ";"
-            "video/x-mp4-part," COMMON_VIDEO_CAPS),
-        GST_STATIC_CAPS (MP123_CAPS "; "
-            AAC_CAPS " ; " AC3_CAPS " ; " ALAC_CAPS " ; " OPUS_CAPS),
-      GST_STATIC_CAPS (TEXT_UTF8)}
+            "video/x-mp4-part," COMMON_VIDEO_CAPS "; "
+            "video/x-av1, " "stream-format = (string) \"obu-stream\", "
+            "alignment = (string) \"tu\", " COMMON_VIDEO_CAPS "; " VP9_CAPS
+            "; "),
+        GST_STATIC_CAPS (MP123_CAPS "; " AAC_CAPS " ; " AC3_CAPS " ; " ALAC_CAPS
+            " ; " OPUS_CAPS),
+        GST_STATIC_CAPS (TEXT_UTF8),
+      GST_STATIC_CAPS_NONE}
   ,
   /* Microsoft Smooth Streaming fmp4/isml */
   /* TODO add WMV/WMA support */
@@ -237,6 +250,7 @@ GstQTMuxFormatProp gst_qt_mux_format_list[] = {
         GST_STATIC_CAPS ("video/quicktime, variant = (string) iso-fragmented"),
         GST_STATIC_CAPS (MPEG4V_CAPS "; " H264_CAPS),
         GST_STATIC_CAPS (MP3_CAPS "; " AAC_CAPS),
+        GST_STATIC_CAPS_NONE,
       GST_STATIC_CAPS_NONE}
   ,
   /* 3GPP Technical Specification 26.244 V7.3.0
@@ -250,7 +264,8 @@ GstQTMuxFormatProp gst_qt_mux_format_list[] = {
         GST_STATIC_CAPS ("video/quicktime, variant = (string) 3gpp"),
         GST_STATIC_CAPS (H263_CAPS "; " MPEG4V_CAPS "; " H264_CAPS),
         GST_STATIC_CAPS (AMR_CAPS "; " MP3_CAPS "; " AAC_CAPS "; " AC3_CAPS),
-      GST_STATIC_CAPS (TEXT_UTF8)}
+        GST_STATIC_CAPS (TEXT_UTF8),
+      GST_STATIC_CAPS_NONE}
   ,
   /* ISO 15444-3: Motion-JPEG-2000 (also ISO base media extension) */
   {
@@ -263,6 +278,7 @@ GstQTMuxFormatProp gst_qt_mux_format_list[] = {
         GST_STATIC_CAPS ("image/x-j2c, " COMMON_VIDEO_CAPS "; "
             "image/x-jpc, " COMMON_VIDEO_CAPS),
         GST_STATIC_CAPS (PCM_CAPS),
+        GST_STATIC_CAPS_NONE,
       GST_STATIC_CAPS_NONE}
   ,
   {
