@@ -27,6 +27,7 @@
 #include <gst/rtp/gstrtpbuffer.h>
 
 #include <string.h>
+#include "gstisomp4elements.h"
 #include "gstrtpxqtdepay.h"
 
 #define MAKE_TLV(a,b)  (((a)<<8)|(b))
@@ -94,6 +95,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 
 #define gst_rtp_xqt_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpXQTDepay, gst_rtp_xqt_depay, GST_TYPE_RTP_BASE_DEPAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpxqtdepay, "rtpxqtdepay",
+    GST_RANK_MARGINAL, GST_TYPE_RTP_XQT_DEPAY, isomp4_element_init (plugin));
 
 static void gst_rtp_xqt_depay_finalize (GObject * object);
 
@@ -129,10 +132,10 @@ gst_rtp_xqt_depay_class_init (GstRtpXQTDepayClass * klass)
   GST_DEBUG_CATEGORY_INIT (rtpxqtdepay_debug, "rtpxqtdepay", 0,
       "QT Media RTP Depayloader");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_xqt_depay_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_xqt_depay_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_xqt_depay_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_xqt_depay_sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP packet depayloader", "Codec/Depayloader/Network",
