@@ -54,6 +54,7 @@ static const QtNodeType qt_node_types[] = {
   {FOURCC_vmhd, "video media information", 0,
       qtdemux_dump_vmhd},
   {FOURCC_smhd, "sound media information", 0},
+  {FOURCC_nmhd, "null media information", 0},
   {FOURCC_gmhd, "base media information header", QT_FLAG_CONTAINER},
   {FOURCC_gmin, "base media info", 0, qtdemux_dump_gmin},
   {FOURCC_dinf, "data information", QT_FLAG_CONTAINER},
@@ -189,6 +190,18 @@ static const QtNodeType qt_node_types[] = {
   {FOURCC_avc3, "AV codec configuration v3", 0},
   {FOURCC_dva1, "AVC-based Dolby Vision derived from avc1", 0},
   {FOURCC_dvav, "AVC-based Dolby Vision derived from avc3", 0},
+  {FOURCC_ai12, "AVC-Intra 100M 1080p25/50", 0},
+  {FOURCC_ai13, "AVC-Intra 100M 1080p24/30/60", 0},
+  {FOURCC_ai15, "AVC-Intra 100M 1080i50", 0},
+  {FOURCC_ai16, "AVC-Intra 100M 1080i60", 0},
+  {FOURCC_ai1p, "AVC-Intra 100M 720p24/30/60", 0},
+  {FOURCC_ai1q, "AVC-Intra 100M 720p25/50", 0},
+  {FOURCC_ai52, "AVC-Intra 50M 1080p25/50", 0},
+  {FOURCC_ai53, "AVC-Intra 50M 1080p24/30/60", 0},
+  {FOURCC_ai55, "AVC-Intra 50M 1080i50", 0},
+  {FOURCC_ai56, "AVC-Intra 50M 1080i60", 0},
+  {FOURCC_ai5p, "AVC-Intra 50M 720p24/30/60", 0},
+  {FOURCC_ai5q, "AVC-Intra 50M 720p25/50", 0},
   {FOURCC_mp4s, "VOBSUB codec configuration", 0},
   {FOURCC_hvc1, "HEVC codec configuration", 0},
   {FOURCC_hev1, "HEVC codec configuration", 0},
@@ -230,6 +243,8 @@ static const QtNodeType qt_node_types[] = {
   {FOURCC_aavd, "AAX encrypted audio", 0},
   {FOURCC_adrm, "AAX DRM key data", 0},
   {FOURCC_vttc, "VTTCueBox 14496-30", QT_FLAG_CONTAINER},
+  {FOURCC_metx, "XML MetaData Sample Entry", 0},
+  {FOURCC_cstb, "Correct Start Time Box", 0},
   {0, "unknown", 0,},
 };
 
