@@ -21,17 +21,17 @@
 
 /**
  * SECTION:element-monoscope
+ * @title: monoscope
  * @see_also: goom
  *
  * Monoscope is an audio visualisation element. It creates a coloured
  * curve of the audio signal like on an oscilloscope.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v audiotestsrc ! audioconvert ! monoscope ! videoconvert ! ximagesink
  * ]|
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -75,6 +75,10 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
 
 #define gst_monoscope_parent_class parent_class
 G_DEFINE_TYPE (GstMonoscope, gst_monoscope, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (monoscope, "monoscope",
+    GST_RANK_NONE, GST_TYPE_MONOSCOPE,
+    GST_DEBUG_CATEGORY_INIT (monoscope_debug, "monoscope", 0,
+        "monoscope element"););
 
 static void gst_monoscope_finalize (GObject * object);
 static GstFlowReturn gst_monoscope_chain (GstPad * pad, GstObject * parent,
@@ -104,10 +108,8 @@ gst_monoscope_class_init (GstMonoscopeClass * klass)
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_monoscope_change_state);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class, &src_template);
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
   gst_element_class_set_static_metadata (gstelement_class, "Monoscope",
       "Visualization",
       "Displays a highly stabilised waveform of audio input",
@@ -165,6 +167,7 @@ gst_monoscope_reset (GstMonoscope * monoscope)
 
   gst_adapter_clear (monoscope->adapter);
   gst_segment_init (&monoscope->segment, GST_FORMAT_UNDEFINED);
+  monoscope->segment_pending = FALSE;
 
   GST_OBJECT_LOCK (monoscope);
   monoscope->proportion = 1.0;
@@ -256,6 +259,10 @@ gst_monoscope_src_negotiate (GstMonoscope * monoscope)
   gst_structure_fixate_field_nearest_int (structure, "width", 320);
   gst_structure_fixate_field_nearest_int (structure, "height", 240);
   gst_structure_fixate_field_nearest_fraction (structure, "framerate", 25, 1);
+  if (gst_structure_has_field (structure, "pixel-aspect-ratio"))
+    gst_structure_fixate_field_nearest_fraction (structure,
+        "pixel-aspect-ratio", 1, 1);
+  target = gst_caps_fixate (target);
 
   gst_monoscope_src_setcaps (monoscope, target);
 
@@ -293,6 +300,7 @@ gst_monoscope_src_negotiate (GstMonoscope * monoscope)
   /* and activate */
   gst_buffer_pool_set_active (pool, TRUE);
 
+  gst_query_unref (query);
   gst_caps_unref (target);
 
   return TRUE;
@@ -314,8 +322,13 @@ ensure_negotiated (GstMonoscope * monoscope)
 
   /* we don't know an output format yet, pick one */
   if (reconfigure || !gst_pad_has_current_caps (monoscope->srcpad)) {
-    if (!gst_monoscope_src_negotiate (monoscope))
-      return GST_FLOW_NOT_NEGOTIATED;
+    if (!gst_monoscope_src_negotiate (monoscope)) {
+      gst_pad_mark_reconfigure (monoscope->srcpad);
+      if (GST_PAD_IS_FLUSHING (monoscope->srcpad))
+        return GST_FLOW_FLUSHING;
+      else
+        return GST_FLOW_NOT_NEGOTIATED;
+    }
   }
   return GST_FLOW_OK;
 }
@@ -341,6 +354,12 @@ gst_monoscope_chain (GstPad * pad, GstObject * parent, GstBuffer * inbuf)
     goto out;
   }
 
+  if (monoscope->segment_pending) {
+    gst_pad_push_event (monoscope->srcpad,
+        gst_event_new_segment (&monoscope->segment));
+    monoscope->segment_pending = FALSE;
+  }
+
   /* don't try to combine samples from discont buffer */
   if (GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_DISCONT)) {
     gst_adapter_clear (monoscope->adapter);
@@ -462,11 +481,16 @@ gst_monoscope_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     case GST_EVENT_SEGMENT:
     {
       /* the newsegment values are used to clip the input samples
-       * and to convert the incomming timestamps to running time so
+       * and to convert the incoming timestamps to running time so
        * we can do QoS */
       gst_event_copy_segment (event, &monoscope->segment);
 
-      res = gst_pad_push_event (monoscope->srcpad, event);
+      /* We forward the event from the chain function after caps are
+       * negotiated. Otherwise we would potentially break the event order and
+       * send the segment event before the caps event */
+      monoscope->segment_pending = TRUE;
+      gst_event_unref (event);
+      res = TRUE;
       break;
     }
     case GST_EVENT_CAPS:
@@ -563,11 +587,8 @@ gst_monoscope_change_state (GstElement * element, GstStateChange transition)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (monoscope_debug, "monoscope", 0,
-      "monoscope element");
+  return GST_ELEMENT_REGISTER (monoscope, plugin);
 
-  return gst_element_register (plugin, "monoscope",
-      GST_RANK_NONE, GST_TYPE_MONOSCOPE);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
