@@ -37,6 +37,9 @@ GST_DEBUG_CATEGORY_STATIC (splitmux_part_debug);
 #define SPLITMUX_PART_TYPE_LOCK(p) g_mutex_lock(&(p)->type_lock)
 #define SPLITMUX_PART_TYPE_UNLOCK(p) g_mutex_unlock(&(p)->type_lock)
 
+#define SPLITMUX_PART_MSG_LOCK(p) g_mutex_lock(&(p)->msg_lock)
+#define SPLITMUX_PART_MSG_UNLOCK(p) g_mutex_unlock(&(p)->msg_lock)
+
 typedef struct _GstSplitMuxPartPad
 {
   GstPad parent;
@@ -51,6 +54,7 @@ typedef struct _GstSplitMuxPartPad
   gboolean is_eos;
   gboolean flushing;
   gboolean seen_buffer;
+  gboolean first_activation;
 
   gboolean is_sparse;
   GstClockTime max_ts;
@@ -136,7 +140,7 @@ handle_buffer_measuring (GstSplitMuxPartReader * reader,
   if (reader->prep_state == PART_STATE_PREPARING_COLLECT_STREAMS &&
       !part_pad->seen_buffer) {
     /* If this is the first buffer on the pad in the collect_streams state,
-     * then calculate inital offset based on running time of this segment */
+     * then calculate initial offset based on running time of this segment */
     part_pad->initial_ts_offset =
         part_pad->orig_segment.start + part_pad->orig_segment.base -
         part_pad->orig_segment.time;
@@ -149,6 +153,9 @@ handle_buffer_measuring (GstSplitMuxPartReader * reader,
   /* Adjust buffer timestamps */
   offset = reader->start_offset + part_pad->segment.base;
   offset -= part_pad->initial_ts_offset;
+  /* We don't add the ts_offset here, because we
+   * want to measure the logical length of the stream,
+   * not to generate output timestamps */
 
   /* Update the stored max duration on the pad,
    * always preferring making DTS contiguous
@@ -159,8 +166,8 @@ handle_buffer_measuring (GstSplitMuxPartReader * reader,
     ts = GST_BUFFER_PTS (buf) + offset;
 
   GST_DEBUG_OBJECT (reader, "Pad %" GST_PTR_FORMAT
-      " incoming PTS %" GST_TIME_FORMAT
-      " DTS %" GST_TIME_FORMAT " offset by %" GST_STIME_FORMAT
+      " incoming DTS %" GST_TIME_FORMAT
+      " PTS %" GST_TIME_FORMAT " offset by %" GST_STIME_FORMAT
       " to %" GST_STIME_FORMAT, part_pad,
       GST_TIME_ARGS (GST_BUFFER_DTS (buf)),
       GST_TIME_ARGS (GST_BUFFER_PTS (buf)),
@@ -225,9 +232,16 @@ splitmux_part_pad_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     return GST_FLOW_FLUSHING;
   }
 
+  if (GST_PAD_LAST_FLOW_RETURN (part_pad->target) == GST_FLOW_NOT_LINKED) {
+    SPLITMUX_PART_UNLOCK (reader);
+    gst_buffer_unref (buf);
+    return GST_FLOW_NOT_LINKED;
+  }
+
   /* Adjust buffer timestamps */
   offset = reader->start_offset + part_pad->segment.base;
   offset -= part_pad->initial_ts_offset;
+  offset += reader->ts_offset;
 
   if (GST_BUFFER_PTS_IS_VALID (buf))
     GST_BUFFER_PTS (buf) += offset;
@@ -267,7 +281,8 @@ splitmux_part_is_eos_locked (GstSplitMuxPartReader * part)
   GList *cur;
   for (cur = g_list_first (part->pads); cur != NULL; cur = g_list_next (cur)) {
     GstSplitMuxPartPad *part_pad = SPLITMUX_PART_PAD_CAST (cur->data);
-    if (!part_pad->is_eos)
+    if (GST_PAD_LAST_FLOW_RETURN (part_pad->target) != GST_FLOW_NOT_LINKED
+        && !part_pad->is_eos)
       return FALSE;
   }
 
@@ -318,14 +333,51 @@ splitmux_is_flushing (GstSplitMuxPartReader * reader)
   return FALSE;
 }
 
+static gboolean
+enqueue_event (GstSplitMuxPartReader * reader, GstSplitMuxPartPad * part_pad,
+    GstEvent * event)
+{
+  gboolean ret = TRUE;
+  GstDataQueueItem *item;
+
+  GST_LOG_OBJECT (reader, "Enqueueing event %" GST_PTR_FORMAT, event);
+  item = g_slice_new (GstDataQueueItem);
+  item->destroy = (GDestroyNotify) splitmux_part_free_queue_item;
+  item->object = GST_MINI_OBJECT (event);
+  item->size = 0;
+  item->duration = 0;
+  if (item->duration == GST_CLOCK_TIME_NONE)
+    item->duration = 0;
+  item->visible = FALSE;
+
+  if (!gst_data_queue_push (part_pad->queue, item)) {
+    splitmux_part_free_queue_item (item);
+    ret = FALSE;
+  }
+
+  return ret;
+}
+
+static gboolean
+resend_sticky_event (GstPad * pad, GstEvent ** event, gpointer user_data)
+{
+  GstSplitMuxPartPad *part_pad = SPLITMUX_PART_PAD_CAST (pad);
+  GstSplitMuxPartReader *reader = part_pad->reader;
+
+  GST_DEBUG_OBJECT (part_pad, "queuing sticky event %" GST_PTR_FORMAT, *event);
+  (void) enqueue_event (reader, part_pad, gst_event_ref (*event));
+
+  return TRUE;
+}
+
 static gboolean
 splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   GstSplitMuxPartPad *part_pad = SPLITMUX_PART_PAD_CAST (pad);
   GstSplitMuxPartReader *reader = part_pad->reader;
   gboolean ret = TRUE;
+  gboolean resend_sticky = FALSE;
   SplitMuxSrcPad *target;
-  GstDataQueueItem *item;
 
   SPLITMUX_PART_LOCK (reader);
 
@@ -356,21 +408,22 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
         goto wrong_segment;
 
       /* Adjust segment */
-      /* Adjust start/stop so the overall file is 0 + start_offset based */
+      /* Adjust start/stop so the overall file is 0 + start_offset based,
+       * adding a fixed offset so that DTS is never negative */
       if (seg->stop != -1) {
         seg->stop -= seg->start;
-        seg->stop += seg->time + reader->start_offset;
+        seg->stop += seg->time + reader->start_offset + reader->ts_offset;
       }
-      seg->start = seg->time + reader->start_offset;
+      seg->start = seg->time + reader->start_offset + reader->ts_offset;
       seg->time += reader->start_offset;
       seg->position += reader->start_offset;
 
-      GST_LOG_OBJECT (pad, "Adjusted segment now %" GST_PTR_FORMAT, event);
-
       /* Replace event */
       gst_event_unref (event);
       event = gst_event_new_segment (seg);
 
+      GST_LOG_OBJECT (pad, "Adjusted segment now %" GST_PTR_FORMAT, event);
+
       if (reader->prep_state != PART_STATE_PREPARING_COLLECT_STREAMS
           && reader->prep_state != PART_STATE_PREPARING_MEASURE_STREAMS)
         break;                  /* Only do further stuff with segments during initial measuring */
@@ -405,6 +458,8 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
           reader->prep_state == PART_STATE_PREPARING_MEASURE_STREAMS) {
         /* Mark this pad as EOS */
         part_pad->is_eos = TRUE;
+        /* Mark the pad to re-send sticky events on the first activation */
+        part_pad->first_activation = TRUE;
         if (splitmux_part_is_eos_locked (reader)) {
           /* Finished measuring things, set state and tell the state change func
            * so it can seek back to the start */
@@ -467,25 +522,22 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
       break;
   }
 
+  /* Re-queue any sticky events on the first activation that were
+   * dropped during probing */
+  if (part_pad->first_activation) {
+    resend_sticky = TRUE;
+    part_pad->first_activation = FALSE;
+  }
+
   /* We are active, and one queue is empty, place this buffer in
    * the dataqueue */
   gst_object_ref (part_pad->queue);
   SPLITMUX_PART_UNLOCK (reader);
 
-  GST_LOG_OBJECT (reader, "Enqueueing event %" GST_PTR_FORMAT, event);
-  item = g_slice_new (GstDataQueueItem);
-  item->destroy = (GDestroyNotify) splitmux_part_free_queue_item;
-  item->object = GST_MINI_OBJECT (event);
-  item->size = 0;
-  item->duration = 0;
-  if (item->duration == GST_CLOCK_TIME_NONE)
-    item->duration = 0;
-  item->visible = FALSE;
+  if (resend_sticky)
+    gst_pad_sticky_events_foreach (pad, resend_sticky_event, NULL);
 
-  if (!gst_data_queue_push (part_pad->queue, item)) {
-    splitmux_part_free_queue_item (item);
-    ret = FALSE;
-  }
+  ret = enqueue_event (reader, part_pad, event);
 
   gst_object_unref (part_pad->queue);
   gst_object_unref (target);
@@ -500,11 +552,11 @@ wrong_segment:
           reader->path, pad));
   return FALSE;
 drop_event:
+  SPLITMUX_PART_UNLOCK (reader);
   GST_LOG_OBJECT (pad, "Dropping event %" GST_PTR_FORMAT
       " from %" GST_PTR_FORMAT " on %" GST_PTR_FORMAT, event, pad, target);
   gst_event_unref (event);
   gst_object_unref (target);
-  SPLITMUX_PART_UNLOCK (reader);
   return TRUE;
 }
 
@@ -631,6 +683,7 @@ gst_splitmux_part_reader_init (GstSplitMuxPartReader * reader)
   g_cond_init (&reader->inactive_cond);
   g_mutex_init (&reader->lock);
   g_mutex_init (&reader->type_lock);
+  g_mutex_init (&reader->msg_lock);
 
   /* FIXME: Create elements on a state change */
   reader->src = gst_element_factory_make ("filesrc", NULL);
@@ -678,6 +731,7 @@ splitmux_part_reader_finalize (GObject * object)
   g_cond_clear (&reader->inactive_cond);
   g_mutex_clear (&reader->lock);
   g_mutex_clear (&reader->type_lock);
+  g_mutex_clear (&reader->msg_lock);
 
   g_free (reader->path);
 
@@ -689,12 +743,12 @@ do_async_start (GstSplitMuxPartReader * reader)
 {
   GstMessage *message;
 
-  GST_STATE_LOCK (reader);
+  SPLITMUX_PART_MSG_LOCK (reader);
   reader->async_pending = TRUE;
 
   message = gst_message_new_async_start (GST_OBJECT_CAST (reader));
   GST_BIN_CLASS (parent_class)->handle_message (GST_BIN_CAST (reader), message);
-  GST_STATE_UNLOCK (reader);
+  SPLITMUX_PART_MSG_UNLOCK (reader);
 }
 
 static void
@@ -702,7 +756,7 @@ do_async_done (GstSplitMuxPartReader * reader)
 {
   GstMessage *message;
 
-  GST_STATE_LOCK (reader);
+  SPLITMUX_PART_MSG_LOCK (reader);
   if (reader->async_pending) {
     message =
         gst_message_new_async_done (GST_OBJECT_CAST (reader),
@@ -712,7 +766,7 @@ do_async_done (GstSplitMuxPartReader * reader)
 
     reader->async_pending = FALSE;
   }
-  GST_STATE_UNLOCK (reader);
+  SPLITMUX_PART_MSG_UNLOCK (reader);
 }
 
 static void
@@ -1246,12 +1300,13 @@ gst_splitmux_part_reader_get_end_offset (GstSplitMuxPartReader * reader)
 
 void
 gst_splitmux_part_reader_set_start_offset (GstSplitMuxPartReader * reader,
-    GstClockTime offset)
+    GstClockTime time_offset, GstClockTime ts_offset)
 {
   SPLITMUX_PART_LOCK (reader);
-  reader->start_offset = offset;
-  GST_INFO_OBJECT (reader, "TS offset now %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (offset));
+  reader->start_offset = time_offset;
+  reader->ts_offset = ts_offset;
+  GST_INFO_OBJECT (reader, "Time offset now %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (time_offset));
   SPLITMUX_PART_UNLOCK (reader);
 }
 
