@@ -26,6 +26,7 @@
 #include <gst/audio/audio.h>
 
 #include <string.h>
+#include "gstrtpelements.h"
 #include "gstrtpmp4adepay.h"
 #include "gstrtputils.h"
 
@@ -60,6 +61,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 #define gst_rtp_mp4a_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpMP4ADepay, gst_rtp_mp4a_depay,
     GST_TYPE_RTP_BASE_DEPAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpmp4adepay, "rtpmp4adepay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_MP4A_DEPAY, rtp_element_init (plugin));
 
 static void gst_rtp_mp4a_depay_finalize (GObject * object);
 
@@ -457,10 +460,3 @@ gst_rtp_mp4a_depay_change_state (GstElement * element,
   }
   return ret;
 }
-
-gboolean
-gst_rtp_mp4a_depay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpmp4adepay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_MP4A_DEPAY);
-}
