@@ -25,6 +25,7 @@
 
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/video/video.h>
+#include "gstrtpelements.h"
 #include "gstrtpsv3vdepay.h"
 #include "gstrtputils.h"
 
@@ -49,8 +50,11 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     );
 
 #define gst_rtp_sv3v_depay_parent_class parent_class
-G_DEFINE_TYPE (GstRtpSV3VDepay, gst_rtp_sv3v_depay,
-    GST_TYPE_RTP_BASE_DEPAYLOAD);
+G_DEFINE_TYPE_WITH_CODE (GstRtpSV3VDepay, gst_rtp_sv3v_depay,
+    GST_TYPE_RTP_BASE_DEPAYLOAD, GST_DEBUG_CATEGORY_INIT (rtpsv3vdepay_debug,
+        "rtpsv3vdepay", 0, "RTP SV3V depayloader"));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpsv3vdepay, "rtpsv3vdepay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_SV3V_DEPAY, rtp_element_init (plugin));
 
 static void gst_rtp_sv3v_depay_finalize (GObject * object);
 
@@ -80,10 +84,10 @@ gst_rtp_sv3v_depay_class_init (GstRtpSV3VDepayClass * klass)
 
   gstelement_class->change_state = gst_rtp_sv3v_depay_change_state;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_sv3v_depay_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_sv3v_depay_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_sv3v_depay_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_sv3v_depay_sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP SVQ3 depayloader", "Codec/Depayloader/Network/RTP",
@@ -265,8 +269,7 @@ gst_rtp_sv3v_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
       avail = gst_adapter_available (rtpsv3vdepay->adapter);
       GST_DEBUG ("Returning completed output buffer [%d bytes]", avail);
       outbuf = gst_adapter_take_buffer (rtpsv3vdepay->adapter, avail);
-      gst_rtp_drop_meta (GST_ELEMENT_CAST (rtpsv3vdepay), outbuf,
-          g_quark_from_static_string (GST_META_TAG_VIDEO_STR));
+      gst_rtp_drop_non_video_meta (rtpsv3vdepay, outbuf);
     }
   }
 
@@ -311,13 +314,3 @@ gst_rtp_sv3v_depay_change_state (GstElement * element,
   }
   return ret;
 }
-
-gboolean
-gst_rtp_sv3v_depay_plugin_init (GstPlugin * plugin)
-{
-  GST_DEBUG_CATEGORY_INIT (rtpsv3vdepay_debug, "rtpsv3vdepay", 0,
-      "RTP SV3V depayloader");
-
-  return gst_element_register (plugin, "rtpsv3vdepay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_SV3V_DEPAY);
-}
