@@ -477,8 +477,12 @@ gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     if (!caps)
       goto no_caps;
 
-    if (gst_rtp_pt_demux_pt_is_ignored (rtpdemux, pt))
+    /* must be after the get_caps() call as get_caps() may cause external code
+     * (e.g. rtpbin) to update the ignored-pt list */
+    if (gst_rtp_pt_demux_pt_is_ignored (rtpdemux, pt)) {
+      gst_clear_caps (&caps);
       goto ignored;
+    }
 
     klass = GST_ELEMENT_GET_CLASS (rtpdemux);
     templ = gst_element_class_get_pad_template (klass, "src_%u");
@@ -563,7 +567,7 @@ invalid_buffer:
     GST_ELEMENT_WARNING (rtpdemux, STREAM, DEMUX, (NULL),
         ("Dropping invalid RTP payload"));
     gst_buffer_unref (buf);
-    return GST_FLOW_ERROR;
+    return GST_FLOW_OK;
   }
 no_caps:
   {
