@@ -33,6 +33,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include "gstudpelements.h"
 #include "gstmultiudpsink.h"
 
 #include <string.h>
@@ -41,6 +42,8 @@
 #include <sys/socket.h>
 #endif
 
+#include <gio/gnetworking.h>
+
 #include "gst/net/net.h"
 #include "gst/glib-compat-private.h"
 
@@ -140,6 +143,8 @@ static guint gst_multiudpsink_signals[LAST_SIGNAL] = { 0 };
 
 #define gst_multiudpsink_parent_class parent_class
 G_DEFINE_TYPE (GstMultiUDPSink, gst_multiudpsink, GST_TYPE_BASE_SINK);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (multiudpsink, "multiudpsink",
+    GST_RANK_NONE, GST_TYPE_MULTIUDPSINK, udp_element_init (plugin));
 
 static void
 gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
@@ -328,7 +333,7 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
    */
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SEND_DUPLICATES,
       g_param_spec_boolean ("send-duplicates", "Send Duplicates",
-          "When a distination/port pair is added multiple times, send packets "
+          "When a destination/port pair is added multiple times, send packets "
           "multiple times as well", DEFAULT_SEND_DUPLICATES,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -1294,14 +1299,26 @@ gst_multiudpsink_start (GstBaseSink * bsink)
       g_object_unref (bind_iaddr);
       family = g_socket_address_get_family (G_SOCKET_ADDRESS (bind_addr));
 
-      if ((sink->used_socket =
-              g_socket_new (family, G_SOCKET_TYPE_DATAGRAM,
-                  G_SOCKET_PROTOCOL_UDP, &err)) == NULL) {
-        g_object_unref (bind_addr);
-        goto no_socket;
+      if (family == G_SOCKET_FAMILY_IPV4) {
+        if ((sink->used_socket =
+                g_socket_new (family, G_SOCKET_TYPE_DATAGRAM,
+                    G_SOCKET_PROTOCOL_UDP, &err)) == NULL) {
+          g_object_unref (bind_addr);
+          goto no_socket;
+        }
+
+        g_socket_bind (sink->used_socket, bind_addr, TRUE, &err);
+      } else {
+        if ((sink->used_socket_v6 =
+                g_socket_new (family, G_SOCKET_TYPE_DATAGRAM,
+                    G_SOCKET_PROTOCOL_UDP, &err)) == NULL) {
+          g_object_unref (bind_addr);
+          goto no_socket;
+        }
+
+        g_socket_bind (sink->used_socket_v6, bind_addr, TRUE, &err);
       }
 
-      g_socket_bind (sink->used_socket, bind_addr, TRUE, &err);
       g_object_unref (bind_addr);
       if (err != NULL)
         goto bind_error;
@@ -1338,10 +1355,9 @@ gst_multiudpsink_start (GstBaseSink * bsink)
   }
 #ifdef SO_SNDBUF
   {
-    socklen_t len;
-    gint sndsize, ret;
+    gint sndsize;
+    GError *opt_err = NULL;
 
-    len = sizeof (sndsize);
     if (sink->buffer_size != 0) {
       sndsize = sink->buffer_size;
 
@@ -1351,24 +1367,22 @@ gst_multiudpsink_start (GstBaseSink * bsink)
        * Linux. */
 
       if (sink->used_socket) {
-        ret =
-            setsockopt (g_socket_get_fd (sink->used_socket), SOL_SOCKET,
-            SO_SNDBUF, (void *) &sndsize, len);
-        if (ret != 0) {
+        if (!g_socket_set_option (sink->used_socket, SOL_SOCKET, SO_SNDBUF,
+                sndsize, &opt_err)) {
           GST_ELEMENT_WARNING (sink, RESOURCE, SETTINGS, (NULL),
-              ("Could not create a buffer of requested %d bytes, %d: %s",
-                  sndsize, ret, g_strerror (errno)));
+              ("Could not create a buffer of requested %d bytes (%s)",
+                  sndsize, opt_err->message));
+          g_clear_error (&opt_err);
         }
       }
 
       if (sink->used_socket_v6) {
-        ret =
-            setsockopt (g_socket_get_fd (sink->used_socket_v6), SOL_SOCKET,
-            SO_SNDBUF, (void *) &sndsize, len);
-        if (ret != 0) {
+        if (!g_socket_set_option (sink->used_socket_v6, SOL_SOCKET, SO_SNDBUF,
+                sndsize, &opt_err)) {
           GST_ELEMENT_WARNING (sink, RESOURCE, SETTINGS, (NULL),
-              ("Could not create a buffer of requested %d bytes, %d: %s",
-                  sndsize, ret, g_strerror (errno)));
+              ("Could not create a buffer of requested %d bytes (%s)",
+                  sndsize, opt_err->message));
+          g_clear_error (&opt_err);
         }
       }
     }
@@ -1377,23 +1391,21 @@ gst_multiudpsink_start (GstBaseSink * bsink)
      * value we set because the kernel allocates extra memory for metadata.
      * The default on Linux is about 100K (which is about 50K without metadata) */
     if (sink->used_socket) {
-      ret =
-          getsockopt (g_socket_get_fd (sink->used_socket), SOL_SOCKET,
-          SO_SNDBUF, (void *) &sndsize, &len);
-      if (ret == 0)
+      if (g_socket_get_option (sink->used_socket, SOL_SOCKET, SO_SNDBUF,
+              &sndsize, NULL)) {
         GST_DEBUG_OBJECT (sink, "have UDP buffer of %d bytes", sndsize);
-      else
+      } else {
         GST_DEBUG_OBJECT (sink, "could not get UDP buffer size");
+      }
     }
 
     if (sink->used_socket_v6) {
-      ret =
-          getsockopt (g_socket_get_fd (sink->used_socket_v6), SOL_SOCKET,
-          SO_SNDBUF, (void *) &sndsize, &len);
-      if (ret == 0)
+      if (g_socket_get_option (sink->used_socket_v6, SOL_SOCKET, SO_SNDBUF,
+              &sndsize, NULL)) {
         GST_DEBUG_OBJECT (sink, "have UDPv6 buffer of %d bytes", sndsize);
-      else
+      } else {
         GST_DEBUG_OBJECT (sink, "could not get UDPv6 buffer size");
+      }
     }
   }
 #endif
