@@ -24,6 +24,10 @@
 #include <unistd.h>             /* for getpid */
 #include "gstosxaudiosink.h"
 
+#if defined(MAC_OS_X_VERSION_MAX_ALLOWED) && MAC_OS_X_VERSION_MAX_ALLOWED < 120000
+#define kAudioObjectPropertyElementMain kAudioObjectPropertyElementMaster
+#endif
+
 static inline gboolean
 _audio_system_set_runloop (CFRunLoopRef runLoop)
 {
@@ -34,7 +38,7 @@ _audio_system_set_runloop (CFRunLoopRef runLoop)
   AudioObjectPropertyAddress runloopAddress = {
     kAudioHardwarePropertyRunLoop,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectSetPropertyData (kAudioObjectSystemObject,
@@ -62,7 +66,7 @@ _audio_system_get_default_device (gboolean output)
   AudioObjectPropertyAddress defaultDeviceAddress = {
     prop_selector,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyData (kAudioObjectSystemObject,
@@ -86,7 +90,7 @@ _audio_system_get_devices (gint * ndevices)
   AudioObjectPropertyAddress audioDevicesAddress = {
     kAudioHardwarePropertyDevices,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyDataSize (kAudioObjectSystemObject,
@@ -126,7 +130,7 @@ _audio_device_is_alive (AudioDeviceID device_id, gboolean output)
   AudioObjectPropertyAddress audioDeviceAliveAddress = {
     kAudioDevicePropertyDeviceIsAlive,
     prop_scope,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyData (device_id,
@@ -138,6 +142,25 @@ _audio_device_is_alive (AudioDeviceID device_id, gboolean output)
   return alive;
 }
 
+static inline gboolean
+_audio_device_is_hidden (AudioDeviceID device_id)
+{
+  OSStatus status = noErr;
+  UInt32 hidden = FALSE;
+  UInt32 property_size = sizeof (hidden);
+  AudioObjectPropertyAddress property_address;
+
+  property_address.mSelector = kAudioDevicePropertyIsHidden;
+
+  status = AudioObjectGetPropertyData (device_id,
+      &property_address, 0, NULL, &property_size, &hidden);
+  if (status != noErr) {
+    return FALSE;
+  }
+
+  return hidden;
+}
+
 static inline guint
 _audio_device_get_latency (AudioDeviceID device_id)
 {
@@ -148,7 +171,7 @@ _audio_device_get_latency (AudioDeviceID device_id)
   AudioObjectPropertyAddress audioDeviceLatencyAddress = {
     kAudioDevicePropertyLatency,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyData (device_id,
@@ -171,7 +194,7 @@ _audio_device_get_hog (AudioDeviceID device_id)
   AudioObjectPropertyAddress audioDeviceHogModeAddress = {
     kAudioDevicePropertyHogMode,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyData (device_id,
@@ -194,7 +217,7 @@ _audio_device_set_hog (AudioDeviceID device_id, pid_t hog_pid)
   AudioObjectPropertyAddress audioDeviceHogModeAddress = {
     kAudioDevicePropertyHogMode,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectSetPropertyData (device_id,
@@ -220,7 +243,7 @@ _audio_device_set_mixing (AudioDeviceID device_id, gboolean enable_mix)
   AudioObjectPropertyAddress audioDeviceSupportsMixingAddress = {
     kAudioDevicePropertySupportsMixing,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   if (AudioObjectHasProperty (device_id, &audioDeviceSupportsMixingAddress)) {
@@ -272,7 +295,7 @@ _audio_device_get_name (AudioDeviceID device_id, gboolean output)
   AudioObjectPropertyAddress deviceNameAddress = {
     kAudioDevicePropertyDeviceName,
     prop_scope,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   /* Get the length of the device name */
@@ -304,7 +327,7 @@ _audio_device_has_output (AudioDeviceID device_id)
   AudioObjectPropertyAddress streamsAddress = {
     kAudioDevicePropertyStreams,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyDataSize (device_id,
@@ -335,7 +358,7 @@ gst_core_audio_audio_device_get_channel_layout (AudioDeviceID device_id,
   AudioObjectPropertyAddress channelLayoutAddress = {
     kAudioDevicePropertyPreferredChannelLayout,
     prop_scope,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   /* Get the length of the default channel layout structure */
@@ -397,7 +420,7 @@ _audio_device_get_streams (AudioDeviceID device_id, gint * nstreams)
   AudioObjectPropertyAddress streamsAddress = {
     kAudioDevicePropertyStreams,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyDataSize (device_id,
@@ -434,7 +457,7 @@ _audio_stream_get_latency (AudioStreamID stream_id)
   AudioObjectPropertyAddress latencyAddress = {
     kAudioStreamPropertyLatency,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyData (stream_id,
@@ -457,7 +480,7 @@ _audio_stream_get_current_format (AudioStreamID stream_id,
   AudioObjectPropertyAddress formatAddress = {
     kAudioStreamPropertyPhysicalFormat,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyData (stream_id,
@@ -480,7 +503,7 @@ _audio_stream_set_current_format (AudioStreamID stream_id,
   AudioObjectPropertyAddress formatAddress = {
     kAudioStreamPropertyPhysicalFormat,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectSetPropertyData (stream_id,
@@ -503,7 +526,7 @@ _audio_stream_get_formats (AudioStreamID stream_id, gint * nformats)
   AudioObjectPropertyAddress formatsAddress = {
     kAudioStreamPropertyAvailablePhysicalFormats,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   status = AudioObjectGetPropertyDataSize (stream_id,
@@ -590,7 +613,7 @@ _audio_stream_change_format (AudioStreamID stream_id,
   AudioObjectPropertyAddress formatAddress = {
     kAudioStreamPropertyPhysicalFormat,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   GST_DEBUG ("setting stream format: " CORE_AUDIO_FORMAT,
@@ -696,7 +719,7 @@ _monitorize_spdif (GstCoreAudio * core_audio)
   AudioObjectPropertyAddress propAddress = {
     kAudioDevicePropertyDeviceHasChanged,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   /* Install the property listener */
@@ -721,7 +744,7 @@ _unmonitorize_spdif (GstCoreAudio * core_audio)
   AudioObjectPropertyAddress propAddress = {
     kAudioDevicePropertyDeviceHasChanged,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMaster
+    kAudioObjectPropertyElementMain
   };
 
   /* Remove the property listener */
@@ -1183,56 +1206,18 @@ done:
 static gboolean
 gst_core_audio_select_device_impl (GstCoreAudio * core_audio)
 {
-  AudioDeviceID *devices = NULL;
   AudioDeviceID device_id = core_audio->device_id;
-  AudioDeviceID default_device_id = 0;
-  gint i, ndevices = 0;
   gboolean output = !core_audio->is_src;
   gboolean res = FALSE;
-#ifdef GST_CORE_AUDIO_DEBUG
-  AudioChannelLayout *channel_layout;
-#endif
-
-  devices = _audio_system_get_devices (&ndevices);
-
-  if (ndevices < 1) {
-    GST_ERROR ("no audio output devices found");
-    goto done;
-  }
-
-  GST_DEBUG ("found %d audio device(s)", ndevices);
-
-#ifdef GST_CORE_AUDIO_DEBUG
-  for (i = 0; i < ndevices; i++) {
-    gchar *device_name;
-
-    if ((device_name = _audio_device_get_name (devices[i], output))) {
-      if (!_audio_device_has_output (devices[i])) {
-        GST_DEBUG ("Input Device ID: %u Name: %s",
-            (unsigned) devices[i], device_name);
-      } else {
-        GST_DEBUG ("Output Device ID: %u Name: %s",
-            (unsigned) devices[i], device_name);
-
-        channel_layout =
-            gst_core_audio_audio_device_get_channel_layout (devices[i], output);
-        if (channel_layout) {
-          gst_core_audio_dump_channel_layout (channel_layout);
-          g_free (channel_layout);
-        }
-      }
-
-      g_free (device_name);
-    }
-  }
-#endif
-
-  /* Find the ID of the default output device */
-  default_device_id = _audio_system_get_default_device (output);
 
   /* Here we decide if selected device is valid or autoselect
    * the default one when required */
   if (device_id == kAudioDeviceUnknown) {
+    AudioDeviceID default_device_id;
+
+    /* Find the ID of the default output device */
+    default_device_id = _audio_system_get_default_device (output);
+
     if (default_device_id != kAudioDeviceUnknown) {
       device_id = default_device_id;
       res = TRUE;
@@ -1240,7 +1225,56 @@ gst_core_audio_select_device_impl (GstCoreAudio * core_audio)
       GST_ERROR ("No device of required type available");
       res = FALSE;
     }
+  } else if (_audio_device_is_hidden (device_id)) {
+    if (_audio_device_is_alive (device_id, output)) {
+      res = TRUE;
+    } else {
+      GST_ERROR ("Requested hidden device not usable");
+      res = FALSE;
+    }
   } else {
+    AudioDeviceID *devices = NULL;
+    gint i, ndevices = 0;
+#ifdef GST_CORE_AUDIO_DEBUG
+    AudioChannelLayout *channel_layout;
+#endif
+
+    devices = _audio_system_get_devices (&ndevices);
+
+    if (ndevices < 1) {
+      GST_ERROR ("no audio output devices found");
+      g_free (devices);
+      return res;
+    }
+
+    GST_DEBUG ("found %d audio device(s)", ndevices);
+
+#ifdef GST_CORE_AUDIO_DEBUG
+    for (i = 0; i < ndevices; i++) {
+      gchar *device_name;
+
+      if ((device_name = _audio_device_get_name (devices[i], output))) {
+        if (!_audio_device_has_output (devices[i])) {
+          GST_DEBUG ("Input Device ID: %u Name: %s",
+              (unsigned) devices[i], device_name);
+        } else {
+          GST_DEBUG ("Output Device ID: %u Name: %s",
+              (unsigned) devices[i], device_name);
+
+          channel_layout =
+              gst_core_audio_audio_device_get_channel_layout (devices[i],
+              output);
+          if (channel_layout) {
+            gst_core_audio_dump_channel_layout (channel_layout);
+            g_free (channel_layout);
+          }
+        }
+
+        g_free (device_name);
+      }
+    }
+#endif
+
     for (i = 0; i < ndevices; i++) {
       if (device_id == devices[i]) {
         res = TRUE;
@@ -1248,18 +1282,17 @@ gst_core_audio_select_device_impl (GstCoreAudio * core_audio)
       }
     }
 
+    g_free (devices);
+
     if (res && !_audio_device_is_alive (device_id, output)) {
       GST_ERROR ("Requested device not usable");
       res = FALSE;
-      goto done;
     }
   }
 
   if (res)
     core_audio->device_id = device_id;
 
-done:
-  g_free (devices);
   return res;
 }
 
